@@ -1,4 +1,4 @@
-from PyQt4 import QtGui, QtCore
+from PySide import QtGui, QtCore
 import enum
 
 class AccountDelegate(QtGui.QItemDelegate):
@@ -6,18 +6,24 @@ class AccountDelegate(QtGui.QItemDelegate):
 		super(AccountDelegate, self).__init__(parent=parent)
 
 	def createEditor(self, parent, option, index):
-
 		lineEdit = QtGui.QLineEdit(parent=parent)
+		pattern = None
 
 		if index.column() in (
 						enum.kAccountTypeColumn_DateField,
 						enum.kAccountTypeColumn_DescriptionField,
 						enum.kAccountTypeColumn_CreditField,
 						enum.kAccountTypeColumn_DebitField):
-			lineEdit.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('[0-9]+'), lineEdit))
+			pattern = QtCore.QRegExp('[0-9]+')
+
+		elif index.column() == enum.kAccountTypeColumn_DateFormat:
+			pattern = QtCore.QRegExp('[dMy/]+')
 
 		elif index.column() == enum.kAccountTypeColumn_CurrencySign:
-			lineEdit.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('-1|1')))
+			pattern = QtCore.QRegExp('-1|1')
+
+		if pattern:
+			lineEdit.setValidator(QtGui.QRegExpValidator(pattern))
 
 		return lineEdit
 
