@@ -1,18 +1,28 @@
+from PySide import QtGui, QtCore, QtSql
 from contextlib  import contextmanager
 import operator
-from PyQt4 import QtGui, QtCore, QtSql
+import re
 
 from version import __VERSION__
 from database import db
 from ui_pydosh import Ui_pydosh
 import utils
-import models 
+import models
 import dialogs
 import stylesheet
 import enum
 import pydosh_rc
+import pdb
 
 class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
+	__operatorMap = {
+		'=':  operator.eq,
+		'>':  operator.gt,
+		'<':  operator.lt,
+		'>=': operator.ge,
+		'<=': operator.le
+	}
+
 	def __init__(self, parent=None):
 		super(PydoshWindow, self).__init__(parent=parent)
 		self.setupUi(self)
@@ -40,8 +50,6 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		self.deleteButton.setEnabled(False)
 		self.removeTagButton.setEnabled(False)
 
-		self.accountCombo.setDefaultText('all')
-
 		self.connectionStatusText.setText('connected to %s@%s' % (db.database, db.hostname))
 
 		amountValidator = QtGui.QRegExpValidator(QtCore.QRegExp("[<>=0-9.]*"), self)
@@ -52,11 +60,6 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 		self.addActions()
 
-		styleSheet = 'QLabel {font: bold;}'
-		self.inTotalLabel.setStyleSheet(styleSheet)
-		self.outTotalLabel.setStyleSheet(styleSheet)
-		self.recordCountLabel.setStyleSheet(styleSheet)
-
 		# Date ranges
 		self.maxInsertDate = None
 		self.populateDates()
@@ -67,7 +70,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		recordModel.setEditStrategy(QtSql.QSqlTableModel.OnFieldChange)
 		recordProxyModel = models.RecordProxyModel(self)
 
-		# "you should not update the source model through 
+		# "you should not update the source model through
 		#  the proxy model when dynamicSortFilter is true"
 		recordProxyModel.setDynamicSortFilter(False)
 		recordProxyModel.setSourceModel(recordModel)
@@ -106,29 +109,17 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		self.tagView.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
 		self.tagView.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
 		self.tagView.horizontalHeader().setDefaultAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
-		self.tagView.setStyleSheet('QTableView {background-color: transparent;}')
 		self.tagView.setShowGrid(False)
 
 		# Set up account model
-		accountModel = models.AccountsModel(self)
-		accountModel.setTable('accounttypes')
-		accountModel.setFilter("""
-			accounttypeid IN (
-				SELECT distinct accounttypeid
-				FROM records
-				WHERE userid=%d)
-			""" % db.userId)
-		accountModel.select()
-		self.accountCombo.setModelColumn(enum.kAccountTypeColumn_AccountName)
-		self.accountCombo.setModel(accountModel)
+		self.accountCombo.setDefaultText('all')
+		self.populateAccounts()
 
 		# Set delays on searchlinededit widgets
 		self.descEdit.setDelay(400)
 		self.amountEdit.setDelay(400)
 
-		#
 		# Set up connections
-		#
 		self.scrolltoEdit.textChanged.connect(self.scrollTo)
 		self.toggleCheckButton.clicked.connect(self.toggleSelected)
 		self.deleteButton.clicked.connect(self.deleteRecords)
@@ -139,11 +130,13 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		recordModel.dataChanged.connect(self.updateTagFilter)
 		recordProxyModel.filterChanged.connect(self.updateTagFilter)
 		recordProxyModel.modelReset.connect(self.updateTagFilter)
-		self.tableView.selectionModel().selectionChanged.connect(self.recordSelectionChanged)
+		selectionModel = self.tableView.selectionModel()
+		selectionModel.selectionChanged.connect(self.recordSelectionChanged)
 		self.tableView.customContextMenuRequested.connect(self.tagEditPopup)
 		tagModel.tagsChanged.connect(self.tagModelChanged)
 		tagModel.selectionChanged.connect(recordProxyModel.setTagFilter)
-		self.tagView.selectionModel().selectionChanged.connect(self.enableRemoveTagButton)
+		selectionModel = self.tagView.selectionModel()
+		selectionModel.selectionChanged.connect(self.enableRemoveTagButton)
 		self.startDateEdit.dateChanged.connect(recordProxyModel.setStartDate)
 		self.endDateEdit.dateChanged.connect(self.endDateChanged)
 		self.endDateEdit.dateChanged.connect(recordProxyModel.setEndDate)
@@ -157,20 +150,16 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 		self.reset()
 
-	def accountSelectionChanged(self, indexes):
+	def accountSelectionChanged(self, items):
 		""" Tell the proxy filter to filter on selected account ids
 		"""
-		model = self.accountCombo.model()
-		accountIds = []
-		for index in indexes:
-			accountIds.append(model.index(index.row(), enum.kAccountTypeColumn_AccountTypeId).data().toPyObject())
-
+		accountIds = [self.accountCombo.itemData(self.accountCombo.findText(item)) for item in items]
 		self.tableView.model().setAccountFilter(accountIds)
 
 	def tagSelectionChanged(self, index):
 		""" Basic tag filter (has tags or not)
 		"""
-		state = self.tagsCombo.itemData(index, QtCore.Qt.UserRole).toPyObject()
+		state = self.tagsCombo.itemData(index, QtCore.Qt.UserRole)
 		if state == enum.kTagCombo_With:
 			value = True
 		elif state == enum.kTagCombo_Without:
@@ -182,7 +171,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 	def checkedSelectionChanged(self, index):
 		""" Set filter for checked records
 		"""
-		state = self.checkedCombo.itemData(index, QtCore.Qt.UserRole).toPyObject()
+		state = self.checkedCombo.itemData(index, QtCore.Qt.UserRole)
 		if state == enum.kCheckedStatus_Checked:
 			value = True
 		elif state == enum.kCheckedStatus_UnChecked:
@@ -194,7 +183,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 	def inOutSelectionChanged(self, index):
 		""" Set filter for in/out amount
 		"""
-		state = self.inoutCombo.itemData(index, QtCore.Qt.UserRole).toPyObject()
+		state = self.inoutCombo.itemData(index, QtCore.Qt.UserRole)
 		if state == enum.kInOutStatus_In:
 			value = True
 		elif state == enum.kInOutStatus_Out:
@@ -205,27 +194,16 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 	def amountFilterChanged(self, text):
 		if text:
-			if text.contains(QtCore.QRegExp('[<>=]+')):
+			if re.match('[<>=]', text):
 				# looks like we have operators, test validity and amount
-				operatorMap = {
-					'=':  operator.eq,
-					'>':  operator.gt,
-					'<':  operator.lt,
-					'>=': operator.ge,
-					'<=': operator.le
-				}
-				rx = QtCore.QRegExp('^(=|>|<|>=|<=)([\\.\\d+]+)')
-				if rx.indexIn(text) != -1 and str(rx.cap(1)) in operatorMap:
-					self.tableView.model().setAmountFilter(rx.cap(2), operatorMap[str(rx.cap(1))])
-				else:
-					# Input not complete yet.
-					return
-			else:
-				# No operator supplied - treat amount as a string
-				self.tableView.model().setAmountFilter(text)
-		else:
-			# No filter
-			self.tableView.model().setAmountFilter(None)
+				match = re.match('^(=|>|<|>=|<=)([\d\.]+)', text)
+				if match:
+					opStr, amount = match.groups()
+					if opStr in self.__operatorMap:
+						self.tableView.model().setAmountFilter(amount, self.__operatorMap[opStr])
+				return
+
+		self.tableView.model().setAmountFilter(text)
 
 	def showAbout(self):
 		""" Show 'about' info
@@ -238,11 +216,11 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 			"<p>Copywrite (c) 2013.</p>"
 			"<p>Written using PyQt %s</p>"
 			"<p><a href=\"http://www.innerhippy.com\">www.innerhippy.com</a></p>"
-			"enjoy!</html>" % (__VERSION__, QtCore.QT_VERSION_STR)
+			"enjoy!</html>" % (__VERSION__, QtCore.__version__)
 			)
 
 	def endDateChanged(self, date):
-		""" End date has changed - if date mode is "previous" month, 
+		""" End date has changed - if date mode is "previous" month,
 			the update start date accordingly
 		"""
 		if self.dateCombo.currentIndex() == enum.kDate_PreviousMonth:
@@ -287,9 +265,9 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 			startDate = QtCore.QDate()
 			endDate = QtCore.QDate()
 
-			# Find the min/max dates for the last import from the proxy model 
+			# Find the min/max dates for the last import from the proxy model
 			for row in xrange(proxyModel.rowCount()):
-				dateForRow = proxyModel.index(row, enum.kRecordColumn_Date).data().toDate()
+				dateForRow = proxyModel.index(row, enum.kRecordColumn_Date).data(QtCore.Qt.UserRole)
 				if not endDate.isValid() or dateForRow > endDate:
 					endDate = dateForRow
 				if not startDate.isValid() or dateForRow < startDate:
@@ -311,9 +289,9 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		"""
 		# try and set last import directory
 		settings = QtCore.QSettings()
-		importDir = settings.value("options/importdirectory").toString()
+		importDir = settings.value("options/importdirectory")
 
-		if importDir.isEmpty():
+		if not importDir:
 			importDir = QtCore.QDir.homePath()
 
 		dialog = QtGui.QFileDialog(self, 'Open File', importDir, "*.csv")
@@ -322,23 +300,27 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		if not dialog.exec_():
 			return
 
-		fileNames = QtCore.QStringList([QtCore.QFileInfo(f).fileName() for f in dialog.selectedFiles()])
+		fileNames = [QtCore.QFileInfo(f).fileName() for f in dialog.selectedFiles()]
 
 		# Save the settings for next time
 		settings.setValue('options/importdirectory', dialog.directory().absolutePath())
 
 		dialog = dialogs.ImportDialog(dialog.selectedFiles(), self)
-		dialog.setWindowTitle(fileNames.join(', '))
+		dialog.setWindowTitle(', '.join(fileNames))
+		dialog.accepted.connect(self.importClosed)
+		dialog.show()
 
-		if dialog.exec_():
-			self.tableView.model().sourceModel().select()
-			self.accountCombo.reset()
-			self.populateDates()
-			self.setDateRange()
+	def importClosed(self):
+		""" Statements have been imported successfully - refresh models
+		"""
+		self.tableView.model().sourceModel().select()
+		self.populateAccounts()
+		self.populateDates()
+		self.setDateRange()
 
 	def recordSelectionChanged(self):
 		""" Only enable buttons if a selection has been made
-		""" 
+		"""
 		enable = len(self.tableView.selectionModel().selectedRows()) > 0
 		self.toggleCheckButton.setEnabled(enable)
 		self.deleteButton.setEnabled(enable)
@@ -351,7 +333,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 		for proxyIndex in self.tableView.selectionModel().selectedRows():
 			index = proxyModel.sourceModel().index(proxyModel.mapToSource(proxyIndex).row(), enum.kRecordColumn_RecordId)
-			recordIds.append(index.data().toPyObject())
+			recordIds.append(index.data())
 		return recordIds
 
 	def enableRemoveTagButton(self):
@@ -392,9 +374,9 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 			if not proxyModel.sourceModel().deleteRecords(indexes):
 				QtGui.QMessageBox.critical(self, 'Database Error',
 					proxyModel.sourceModel().lastError().text(), QtGui.QMessageBox.Ok)
-			
+
 			# Finally, re-populate accounts and date range in case this has changed
-			self.accountCombo.reset()
+			self.populateAccounts()
 			self.populateDates()
 			self.setDateRange()
 
@@ -424,28 +406,45 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 					self.tableView.selectRow(proxyModel.mapFromSource(match[0]).row())
 
 			self.tableView.setSelectionMode(selectionMode)
-			
+
 			selected = self.tableView.selectionModel().selectedRows()
 
 			# Scroll to first selected row
 			if selected:
 				self.tableView.scrollTo(selected[0], QtGui.QAbstractItemView.EnsureVisible)
 
+	def populateAccounts(self):
+		""" Populate account data for the current user
+		"""
+		self.accountCombo.clear()
+		query = QtSql.QSqlQuery("""
+		    SELECT accounttypeid, accountname
+		      FROM accounttypes
+		     WHERE accounttypeid IN (
+		           SELECT DISTINCT accounttypeid
+		                      FROM records
+		                     WHERE userid=%s)
+		  ORDER BY accountname
+		""" % db.userId)
+
+		while query.next():
+			self.accountCombo.addItem(query.value(1), query.value(0))
+
 	def populateDates(self):
 		""" Set date fields to min and max values
 		"""
 		query = QtSql.QSqlQuery("""
-			SELECT MIN(date), 
-				   MAX(date),  
+			SELECT MIN(date),
+				   MAX(date),
 				   MAX(insertdate)
 			  FROM records
 			 WHERE userid=%d
 			""" % db.userId)
 
 		if query.next():
-			startDate = query.value(0).toDate()
-			endDate = query.value(1).toDate()
-			maxInsertDateTime = query.value(2).toDateTime()
+			startDate = query.value(0)
+			endDate = query.value(1)
+			maxInsertDateTime = query.value(2)
 
 			self.startDateEdit.setDateRange(startDate, endDate)
 			self.endDateEdit.setDateRange(startDate, endDate)
@@ -484,7 +483,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		with utils.signalsBlocked(signalsToBlock):
 			self.tagView.model().sourceModel().clearSelection()
 			self.tableView.model().clearFilters()
-			
+
 			self.dateCombo.setCurrentIndex(enum.kDate_PreviousMonth)
 
 			# Signals blocked so need to reset filter manually
@@ -550,14 +549,14 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 		model = self.tableView.model()
 		for row in xrange(model.rowCount()):
-			amount = model.index(row, enum.kRecordColumn_Amount).data(QtCore.Qt.UserRole).toPyObject()
+			amount = model.index(row, enum.kRecordColumn_Amount).data(QtCore.Qt.UserRole)
 			if amount > 0.0:
 				inTotal += amount
 			else:
 				outTotal += abs(amount)
 
-		self.inTotalLabel.setText(QtCore.QString("%L1").arg(inTotal, 0, 'f', 2))
-		self.outTotalLabel.setText(QtCore.QString("%L1").arg(outTotal, 0, 'f', 2))
+		self.inTotalLabel.setText('%.2f' % inTotal)
+		self.outTotalLabel.setText('%.2f' % outTotal)
 		self.recordCountLabel.setText('%d / %d' % (model.rowCount(), self.tableView.model().sourceModel().rowCount()))
 
 	def tagModelChanged(self):
@@ -575,7 +574,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		model = self.tableView.model()
 
 		for i in xrange(model.rowCount()):
-			recordIds.append(model.index(i, enum.kRecordColumn_RecordId).data().toPyObject())
+			recordIds.append(model.index(i, enum.kRecordColumn_RecordId).data())
 
 		self.tagView.model().sourceModel().setRecordFilter(recordIds)
 		self.tagView.resizeColumnsToContents()
@@ -597,13 +596,14 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 		# Assign selected records with the new tag
 		tagId = proxyModel.sourceModel().addTag(tagName)
 		proxyModel.sourceModel().addRecordTags(tagId, self.selectedRecordIds())
+		self.tagView.resizeColumnsToContents()
 
 	def removeTag(self):
 		""" Delete a tag - ask for confirmation if tag is currently assigned to records
 		"""
 		proxyModel = self.tagView.model()
 		for proxyIndex in self.tagView.selectionModel().selectedRows():
-			assignedRecords = proxyModel.sourceModel().index(proxyModel.mapToSource(proxyIndex).row(), enum.kTagsColumn_RecordIds).data().toPyObject()
+			assignedRecords = proxyModel.sourceModel().index(proxyModel.mapToSource(proxyIndex).row(), enum.kTagsColumn_RecordIds).data()
 			if assignedRecords:
 				if QtGui.QMessageBox.question(
 						self, 'Delete Tags',
@@ -611,9 +611,11 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 						QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) != QtGui.QMessageBox.Yes:
 					continue
 			with utils.showWaitCursor():
-				tagId = proxyModel.sourceModel().index(proxyModel.mapToSource(proxyIndex).row(), enum.kTagsColumn_TagId).data().toPyObject()
+				tagId = proxyModel.sourceModel().index(proxyModel.mapToSource(proxyIndex).row(), enum.kTagsColumn_TagId).data()
 				proxyModel.sourceModel().removeTag(tagId)
 
+		self.tagView.resizeColumnsToContents()
+
 	def tagEditPopup(self, pos):
 		self.tableView.viewport().mapToGlobal(pos)
 
@@ -623,10 +625,11 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 
 		selectedRecordIds = set(self.selectedRecordIds())
 		model = self.tagView.model()
+
 		for row in xrange(model.rowCount()):
-			tagId = model.index(row, enum.kTagsColumn_TagId).data().toPyObject()
-			tagName = model.index(row, enum.kTagsColumn_TagName).data().toPyObject()
-			tagRecordIds = model.index(row, enum.kTagsColumn_RecordIds).data().toPyObject()
+			tagId = model.index(row, enum.kTagsColumn_TagId).data()
+			tagName = model.index(row, enum.kTagsColumn_TagName).data()
+			tagRecordIds = model.index(row, enum.kTagsColumn_RecordIds).data()
 
 			item = QtGui.QListWidgetItem(tagName)
 			item.setData(QtCore.Qt.UserRole, tagId)
@@ -643,23 +646,31 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 			else:
 				item.setCheckState(QtCore.Qt.Unchecked)
 				tooltip = 'no selected records have tag %r' % str(tagName)
-			
+
 			item.setData(QtCore.Qt.ToolTipRole, tooltip)
 			tagList.addItem(item)
 
-		action = QtGui.QWidgetAction(self)
-		action.setDefaultWidget(tagList)
+		if tagList.count():
+			action = QtGui.QWidgetAction(self)
+			action.setDefaultWidget(tagList)
+			menu = QtGui.QMenu(self)
+			menu.addAction(action)
+			menu.exec_(self.tableView.viewport().mapToGlobal(pos))
 
-		menu = QtGui.QMenu(self)
-		menu.addAction(action)
-		menu.exec_(self.tableView.viewport().mapToGlobal(pos))
+			if not self.selectedRecordIds():
+				if not self.amountEdit.text() and self.descEdit.text():
+					self.descEdit.clear()
+					self.descEdit.setFocus(QtCore.Qt.OtherFocusReason)
+				elif not self.descEdit.text() and self.amountEdit.text():
+					self.amountEdit.clear()
+					self.amountEdit.setFocus(QtCore.Qt.OtherFocusReason)
 
 	@utils.showWaitCursorDecorator
 	def saveTagChanges(self, item):
 		""" Save tag changes to tag model
 		"""
-		checkState = item.data(QtCore.Qt.CheckStateRole).toPyObject()
-		tagId = item.data(QtCore.Qt.UserRole).toPyObject()
+		checkState = item.data(QtCore.Qt.CheckStateRole)
+		tagId = item.data(QtCore.Qt.UserRole)
 
 		if checkState == QtCore.Qt.Unchecked:
 			self.tagView.model().sourceModel().removeRecordTags(tagId, self.selectedRecordIds())
@@ -670,6 +681,8 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 			# If there's no selection available then close the tag menu
 			item.listWidget().parent().close()
 
+		self.displayRecordCount()
+
 	def scrollTo(self, text):
 		# Tell the model to highlight or un-highlight matching rows
 		self.tableView.model().sourceModel().highlightText(text)
@@ -683,7 +696,7 @@ class PydoshWindow(Ui_pydosh, QtGui.QMainWindow):
 class TagListWidget(QtGui.QListWidget):
 	""" Simple extension to QListWidget to allow persistence of editor
 		when used in QMenu popup
-	""" 
+	"""
 	def __init__(self, parent=None):
 		super(TagListWidget, self).__init__(parent=parent)
 		self.__persistEditor = False
@@ -692,4 +705,4 @@ class TagListWidget(QtGui.QListWidget):
 		self.__persistEditor = persist
 
 	def persistEditor(self):
-		return self.__persistEditor
\ No newline at end of file
+		return self.__persistEditor
