@@ -1,11 +1,13 @@
 import sys
-from PyQt4 import QtGui, QtCore
+from PySide import QtGui, QtCore
 from dialogs import LoginDialog
 from main import PydoshWindow
 import stylesheet
-#QtCore.pyqtRemoveInputHook()
 import pydosh_rc
 
+# from PyQt4.QtCore import pyqtRemoveInputHook
+# pyqtRemoveInputHook()
+
 def main():
 
 	app = QtGui.QApplication(sys.argv)
